/*
 * GraphBOM.java
 *
 * Created on 18 de marzo de 2005, 22:28
 */

package oracle;

import java.awt.Point;
import java.util.ArrayList;

/**
 * La clase GraphBOM crea un vector de NodeBOM de tamao size+1, el primer nodo
 * siempre ser el nodo 0 por lo que se accedern el resto de nodos desde la
 * posicin 1..size. El nombre de los nodos es su posicin en el GraphBOM.
 *
 * @author Alejandro Touza Paredes
 */
public class FactorOracle{
    
    final static int NODO_INICIAL = 0;
    private ArrayList nodes;
    private int[] patternsFinals;
    private int size;
    private int numberOfPatterns;
    private String pattern, patternTwo, patternThree;
    private String patternAux;
    private String currentWord;
    private int currentStep;
    private int currentNode;
    private int currentLinkNode;
    private int numberOfLinks;
    private int numberOfTransitions;
    private int patternInTreatment;
    private int charPatternInTreatment;
    private boolean isPatternContained;
    
    //CONSTRUCTORAS
    /** Crea una nueva instancia de GraphOracle. */
    public FactorOracle(String pattern) {
        
        this.pattern = pattern;
        patternAux = pattern;
        initializeVars(1);
        
     }
    
    /** Crea una nueva instancia de GraphOracle con 2 patterns. */
    public FactorOracle(String pattern, String patternTwo) {
        
        this.pattern = pattern;
        patternAux = pattern;
        this.patternTwo = patternTwo;
        initializeVars(2);
     }
    
    /** Crea una nueva instancia de GraphOracle con 3 patterns. */
    public FactorOracle(String pattern, String patternTwo, String patternThree) {
        
        this.pattern = pattern;
        patternAux = pattern;
        this.patternTwo = patternTwo;
        this.patternThree = patternThree;
        initializeVars(3);
     }
    
    public void initializeVars(int numberOfPatterns){
        
        isPatternContained = false;
        patternsFinals = new int[3];
        for(int i=0; i<3; i++) patternsFinals[i]=0;
        currentStep = FactorOracleConst.STEP_0;
        currentNode = 1;
        currentLinkNode = 0;
        numberOfLinks = 0;
        numberOfTransitions = 0;
        this.numberOfPatterns = numberOfPatterns;
        patternInTreatment = 1;
        charPatternInTreatment = 0;
        size = patternAux.length()+1;
        currentWord = pattern.substring(charPatternInTreatment, charPatternInTreatment+1);
        nodes = new ArrayList();
        /*//////////////////////////////////////////////////////////////
        //CUUUUUUTRE
        int totalL = pattern.length()+1; 
        switch(this.numberOfPatterns)
        {
            case 3: totalL += patternThree.length(); 
            case 2: totalL += patternTwo.length();
        }
        for(int i=0; i<totalL ; i++) nodes.add(new NodeOracle(i));
        ///////////////////////////////////////////////////////////////*/
    this.print();
        for(int i=0; i<size; i++) nodes.add(new NodeOracle(i));
        //for(int i=0; i<size; i++) getNode(i).setName((size-i-1)+"");
        /*/for(int i=0; i<pattern.length(); i++) getNode(i+1).setName(pattern.substring(i, i+1));
        nodes[0].setName("-");//*/
        
        getNode(0).setType(FactorOracleConst.NODE_INITIAL_FINAL);
    }
    //CONSULTORAS
    /** Devuelve el tamao del grafo.
     *  @return El tamao del grafo */
    public int size(){
        
        return size;
    }
    /** Devuelve el nmero de patterns con los que se forma el autmata.
     *  @ return El nmero de patterns con los que se forma el autmata. */
    public int numberOfPatterns(){
        
        return numberOfPatterns;
    }
    /** Devuelve el pattern con el que se ha construido el grafo.
     *  @return El pattern del grafo */
    public String pattern(){
        
        return pattern;
    }
    /** Devuelve el 2 pattern con el que se ha construido el grafo si lo tuviere.
     *  @return El 2 pattern del grafo si lo tuviere.*/
    public String patternTwo(){
        
        return patternTwo;
    }
    /** Devuelve el 3er pattern con el que se ha construido el grafo si lo tuviere.
     *  @return El 3er pattern del grafo si lo tuviere.*/
    public String patternThree(){
        
        return patternThree;
    }
    /** Decuelve el vector que nos indica, para cada pattern, cual es su nodo final
     *  en el autmata Factor Oracle creado. */
    public int patternsFinals(int i){
        return patternsFinals[i];
    }
    /**
     * Devuelve el Nodo i del grafo.
     * @return El nodo i del grafo.
     * @param i 
     * @throws java.lang.ArrayIndexOutOfBoundsException 
     */
    public NodeOracle getNode(int i){
         
        return ((NodeOracle)nodes.get(i));
    }
    /** Indica si el grafo ya se ha completado. Devuelve true si ya se ha acabado.*/
    public boolean isEnded(){
        
        return currentStep==FactorOracleConst.STEP_END;
    }
    /** Nos devuelve el currentNode. 0 si el grafo ya est completo */
    public int getCurrentNodeValue(){
        
        return currentNode;
    }
    /** Nos devuelve el currentNode. Null si el grafo ya est completo */
    public NodeOracle getCurrentNode(){
        
        if (currentStep==FactorOracleConst.STEP_END) return null;
        return getNode(currentNode);
    }
    public int getCurrentStep(){
        
        return currentStep;
    }
    public int getNumberOfTransitions(){
        
        return numberOfTransitions;
    }
    public int getNumberOfLinks(){
        
        return numberOfLinks;
    }
    public int getTotalTransitions(){
        
        return numberOfLinks + numberOfTransitions;
    }
    public void print(){
         
        System.out.println("CONTENIDO DEL GRAFO: " + this.pattern);
        System.out.println(".............");
        for(int i=0; i<nodes.size(); i++)
        {
            if (!getNode(i).isEmpty()) getNode(i).print();
        }
        System.out.println(".............");
    }
    public void printPatternsFinals(){
         
        System.out.println("[" + patternsFinals[0] + 
                           "," + patternsFinals[1] + 
                           "," + patternsFinals[2] + 
                           "]");
    }
    /** Nos indica cuantos caracteres de search son aceptados por el grafo i el ltimo nodo
     *  visitado Point(x, y) --> (x - caracteres aceptados, y - ltimo nodo visitado).
     *  @return Point(x, y) --> (x - caracteres aceptados, y - ltimo nodo visitado).
     *  @param search String que queremos que acepte el grafo.
     *  @throws java.lang.NullPointerException si <b>search</b> es nulo.*/
    public Point search(String search)throws NullPointerException{
        
        int i = 0, j = 0, k = 0;
        
        //System.out.println("------------------------------------------------------");
        //System.out.println("CERCA GRAFO(" + pattern + "): " + search);
        for(i = 0; i<search.length() && (k = getNode(j).hasTransition(search.substring(i, i+1)))!=-1; i++)
        {
            j = k; //El nodo actual pasa a ser el nodo destino de la transicin buscada
        }
        
        //System.out.println("El grafo acepta " + i + " caracteres de " + search + " es decir, (" +
        //                   search.substring(0, i) + ")");
        
        return new Point(i, j);
    }

    //MODIFICADORAS
    /** Realiza un paso del Factor Oracle y devuelve la transicin o suffix-link
     *  explorada o aadida en dicho paso. */
    public Transition step(){
        
        //t es la transicin que retornaremos, puede ser tanto una nueva creada o una
        //existente
        Transition t = new Transition();
        int linkEnd;
        int nodoDestino;
        
        switch(currentStep){
            
            //Es el paso donde avanzamos un nodo del autmata, por lo tanto SIEMPRE aadiremos
            //una transicin al autmata a no ser que se trate de un autmata con varios patterns
            //en ese caso, el 2 y siguientes patterns deberan comprobar si ya existe una transicin
            //que satisface el carcter a tratar. Tener en cuenta que la 1 nueva transicin creada
            //por los sucesivos patterns han de situarse como nodo size + 1.
            case FactorOracleConst.STEP_0: 
                                    t.setValues(currentLinkNode, currentNode, currentWord);
                                    getNode(currentLinkNode).putTransition(t);
                                    numberOfTransitions++;
                                    currentStep = FactorOracleConst.STEP_1;
                                    break;
            case FactorOracleConst.STEP_1: 
                                    if(currentLinkNode==NODO_INICIAL)
                                    {
                                        getCurrentNode().setLink(currentNode, NODO_INICIAL);
                                        numberOfLinks++;
                                        t = getCurrentNode().getLink();
                                        currentStep = FactorOracleConst.STEP_0;
                                        currentLinkNode = currentNode;
                                        currentNode++;
                                        updateCurrentValues();
                                    }else
                                    {
                                        t = getNode(currentLinkNode).getLink();
                                        currentStep = FactorOracleConst.STEP_2;
                                    }
                                    break;
            case FactorOracleConst.STEP_2: 
                                    linkEnd = getNode(currentLinkNode).getLinkEnd();
                                    nodoDestino = getNode(linkEnd).hasTransition(currentWord);
                                    if (nodoDestino!=-1)
                                    {
                                        //Aqu hay que coger la transicin que ya existe
                                        t.setValues(linkEnd, nodoDestino, currentWord);
                                        currentStep = FactorOracleConst.STEP_3;
                                    }
                                    else
                                    {
                                        t.setValues(linkEnd, currentNode, currentWord);
                                        getNode(linkEnd).putTransition(t);
                                        numberOfTransitions++;
                                        //Equivalente a la llamada recursiva
                                        currentLinkNode = linkEnd;
                                        currentStep = FactorOracleConst.STEP_1;
                                    }
                                    break;
           case FactorOracleConst.STEP_3:  
                                    //Sabemos que la transicion existe as que aadimos el suffix-link
                                    linkEnd = getNode(currentLinkNode).getLinkEnd();
                                    nodoDestino = getNode(linkEnd).hasTransition(currentWord);
                                    getCurrentNode().setLink(currentNode, nodoDestino);
                                    numberOfLinks++;
                                    // Cogemos el suffix-link
                                    t = getCurrentNode().getLink();
                                    currentStep = FactorOracleConst.STEP_0;
                                    currentLinkNode = currentNode;
                                    currentNode++;
                                    updateCurrentValues();
                                    break;
                                    
            /////////////////////////////////////////////////////////////////////////////////////                        
            //Para el 2 y sucesivos patterns                        
            case FactorOracleConst.STEP_0B:
                                    isPatternContained = false;
                                    nodoDestino = getNode(currentNode).hasTransition(currentWord);
                                    //Hemos encontrado una transicin con currentWord ya creada
                                    if (nodoDestino!=-1)
                                    {
                                        t.setValues(currentNode, nodoDestino, currentWord);
                                        //Si seguimos la transicin desde este paso deberiamos
                                        //seguir comprobando si hay otras ms
                                        currentStep = FactorOracleConst.STEP_0B;
                                        //No usado en este punto - currentLinkNode = currentNode;
                                        currentNode = nodoDestino;
                    System.out.println("nodoDestino: " + nodoDestino);
                                        isPatternContained = true;
                                        updateCurrentValues();
                                    }
                                    else
                                    {
                                        //este size creo que solo vale pa la 1 vez, en principio
                                        //har falta actualizar size y no volveremos a pasar por
                                        //este paso hasta que no cojamos el siguiente pattern
                                        t.setValues(currentNode, size, currentWord);
                                        //hay que aadir tantos nodos como caracteres le queden
                                        //a pattern aux por poner
                   System.out.println("Size: " + size);
                                        nodes.add(new NodeOracle(size));
                                        getNode(currentNode).putTransition(t);
                                        numberOfTransitions++;
                                        //hace falta crear ahora los suffix-link
                                        //Equivalente a la llamada recursiva
                                        //currentLinkNode = linkEnd;
                                        //malamente, en el STEP_3 no se aaden ms nodos
                                        //al arraylist, pero llegados aqu deberamos
                                        //saber que se aadiran tantos nodos como caracteres
                                        //le queden a patternAux
                                        currentLinkNode = currentNode;
                                        currentNode = size;
                                        size++;
                                        //Aqu hace falta meter los suffix-link
                                        currentStep = FactorOracleConst.STEP_1B; 
                                        isPatternContained = false;
                                        //currentStep = FactorOracleConst.STEP_0B;
                                        //updateCurrentValues();
                                    }
                                    break;   
            //Creamos un link al Nodo Inicial  pasamos al siguiente paso                        
            case FactorOracleConst.STEP_1B:  
                                    isPatternContained = false;
                                    if(currentLinkNode==NODO_INICIAL)
                                    {
                                        getCurrentNode().setLink(currentNode, NODO_INICIAL);
                                        numberOfLinks++;
                                        t = getCurrentNode().getLink();
                                        currentStep = FactorOracleConst.STEP_0B;
                                        currentLinkNode = currentNode;
                                        //currentNode++;
                                        updateCurrentValues();
                                    }else
                                    {
                                        t = getNode(currentLinkNode).getLink();
                                        currentStep = FactorOracleConst.STEP_2B;
                                    }
                                    break;
             //Seguimos los suffix-link                        
             case FactorOracleConst.STEP_2B: 
                                    isPatternContained = false;
                                    linkEnd = getNode(currentLinkNode).getLinkEnd();
                                    nodoDestino = getNode(linkEnd).hasTransition(currentWord);
                                    //Si ya existe la transicin la mostramos y vamos al 
                                    //paso 3B para crear el suffix link a ese nodo
                                    if (nodoDestino!=-1)
                                    {
                                        //Aqu hay que coger la transicin que ya existe
                                        t.setValues(linkEnd, nodoDestino, currentWord);
                                        currentStep = FactorOracleConst.STEP_3B;
                                    }
                                    //Creamos la transicin al nodoActual y seguimos
                                    //el suffix-link
                                    else
                                    {
                                        t.setValues(linkEnd, currentNode, currentWord);
                                        getNode(linkEnd).putTransition(t);
                                        numberOfTransitions++;
                                        //Equivalente a la llamada recursiva
                                        currentLinkNode = linkEnd;
                                        currentStep = FactorOracleConst.STEP_1B;
                                    }
                                    break;
           case FactorOracleConst.STEP_3B:  
                                    isPatternContained = false;
                                    //Sabemos que la transicion existe as que aadimos el suffix-link
                                    linkEnd = getNode(currentLinkNode).getLinkEnd();
                                    nodoDestino = getNode(linkEnd).hasTransition(currentWord);
                                    getCurrentNode().setLink(currentNode, nodoDestino);
                                    numberOfLinks++;
                                    // Cogemos el suffix-link
                                    t = getCurrentNode().getLink();
                                    currentStep = FactorOracleConst.STEP_0B;
                                    currentLinkNode = currentNode;
                                    updateCurrentValues();
                                    break;
        }
        
        return t;
    }
    
    private void updateCurrentValues(){
        
    /*System.out.println("currentNode < size?: " + currentNode + " < " + size);
    */    
        //Comprobamos si ya hemos recorrido todo el pattern para saber si hemos acabado con este
        if(charPatternInTreatment + 1 < patternAux.length())
        {
            charPatternInTreatment++;
            //Necesitamos ver con que pattern estamos tratando para la currentWord
            currentWord = patternAux.substring(charPatternInTreatment, charPatternInTreatment+1);
        }else
        {
            //Hara falta comprobar de alguna manera si vamos a buscar otra palabra. Quizs
            //seria necesaria otra funcin que sera conocida desde el exterior para aadir
            //otra cadena al autmata actual.
     /*System.out.println("numberOfPatterns == patternInTreatment?: " 
                               + numberOfPatterns + " == " + patternInTreatment);
     */
            if(numberOfPatterns == patternInTreatment)
            {    
                currentStep = FactorOracleConst.STEP_END;
                //Indicamos el nodo Final del autmata para ese pattern
                if(!isPatternContained) patternsFinals[patternInTreatment-1] = size-1;
                else patternsFinals[patternInTreatment-1] = currentNode;
                //Si solo hay un pattern aqu se acaba todo
                patternInTreatment = 0;
            }
            else
            {
                //Indicamos el nodo Final del autmata para ese pattern
                if(!isPatternContained) patternsFinals[patternInTreatment-1] = size-1;
                else patternsFinals[patternInTreatment-1] = currentNode;
                
                //Empezamos a aadir el siguiente pattern
                patternInTreatment++;
                switch(patternInTreatment)
                {
                    case 2: patternAux = patternTwo;
                            break;
                    case 3: patternAux = patternThree;
                }
                charPatternInTreatment = 0;
                currentStep = FactorOracleConst.STEP_0B;
                //Necesitamos ver con que pattern estamos tratando para la currentWord
                currentWord = patternAux.substring(charPatternInTreatment, charPatternInTreatment+1);
            }
            if(numberOfPatterns>1)
            {
                if(!isPatternContained) getNode(size-1).setType(FactorOracleConst.NODE_FINAL_SPECIAL);
                else getNode(currentNode).setType(FactorOracleConst.NODE_FINAL_SPECIAL);
            }
            
            //Esto para empezar a mirar desde el principio del autmata cuando cambiemos de pattern
            currentNode = 0;
            currentLinkNode = 0;
        }
     /*System.out.println("patternAux: " + patternAux);
       System.out.println("charPatternInTreatment: " + charPatternInTreatment);
       System.out.println("CurrentWord: " + currentWord);
       System.out.println("-------------------------------------------");
     */
    }
    
    /** Aade nuevas cadenas a la/s ya existente/s en el autmata. */
    public void addOracle(String newPattern, int numPattern){
        
        //p.x = n coincidencias, p.y = ltimo nodo visitado
        //Buscamos el pattern a introducir en el autmata actual
        Point p = search(newPattern);
        
        //Si la longitud del nuevo pattern es diferente del nmero de coincidencias encontradas
        //es que el autmata actual no reconoce todos los caracteres de nuestra cadena.
        // (He cambiado el < por != porque en realidad nunca ser mayor).
        if(p.x!=newPattern.length())
        {
            //Todo este rollo hay que pasarlo a pasos.............................................
            //Aadimos todos los nodos necesarios para poder reconocer el nuevo pattern en nuestro
            //autmata. Longitud del pattern - coincidencias ya encontradas
            for(int i=0; i<newPattern.length()-p.x; i++)nodes.add(new NodeOracle(size+i));
            
            currentWord = newPattern.substring(p.x, p.x+1);
            //ponemos la 1 transicin desde el ltimo nodo visitado en la bsqueda (p.y) hasta
            //el primer nuevo nodo creado (size)
            getNode(p.y).putTransition(new Transition(p.y, size, currentWord));
            numberOfTransitions++;
            recursiveOracle(p.y, size);
            p.x++;
            for(int i=0; i<newPattern.length()-(p.x); i++)
            {
                currentWord = newPattern.substring(p.x+i, p.x+i+1);
                getNode(size+i).putTransition(new Transition(size+i, size+i+1, currentWord));
                numberOfTransitions++;
                recursiveOracle(size+i, size+i+1);
            }
            getNode(size + newPattern.length() - p.x).setType(FactorOracleConst.NODE_FINAL_SPECIAL);
            size = nodes.size();
            patternsFinals[numPattern-1] = size-1;
        }
        //Nuestro pattern ya est contenido en el autmata actual, solo necesitamos marcar
        //el ltimo nodo visitado como especial.
        else
        {
            getNode(p.y).setType(FactorOracleConst.NODE_FINAL_SPECIAL);
            patternsFinals[numPattern-1] = p.y;
        }
    }
    /** Construye el grafo indicado por pattern segn la tcnica de Factor Oracle.*/
    public void Oracle(){
        
        for(int i=0; i<pattern.length(); i++)
        {
            currentWord = pattern.substring(i, i+1);
            //aadimos la transicin (i) ----(pattern[i])----> (i+1)
            getNode(i).putTransition(new Transition(i, i+1, currentWord));
            numberOfTransitions++;
            recursiveOracle(i, i+1);
        }
        //Si el Factor Oracle es mltiple ponemos la etiqueta "1"
        if(numberOfPatterns>1)
        {
            getNode(size-1).setType(FactorOracleConst.NODE_FINAL_SPECIAL);
            patternsFinals[0] = size-1;
        }
        if(patternTwo  !=null) addOracle(patternTwo,   2);
        if(patternThree!=null) addOracle(patternThree, 3);
        
        currentStep = FactorOracleConst.STEP_END;
 //////////////////////////CAMBIAR ESTO PQ ES MENTIRA!!!!!!!
        numberOfLinks = pattern.length();
    }
    //PRIVADAS
    /** Realiza los pasos recursivos del Factor Oracle para encontrar el suffix-link
     *  del nodo actual. */
    private void recursiveOracle(int i, int nodoActual){
        //Miramos el suffix-link a no ser que sea el nodo inicial. Si lo es hacemos que apunte a l
        if(i==NODO_INICIAL) getNode(nodoActual).setLink(nodoActual, NODO_INICIAL); 
            
        //Si no es el nodo inicial seguimos el suffix-link del nodes[i]
        else
        {
            //Cogemos el Nodo destino del suffix link de nodes[i]
            int linkEnd = getNode(i).getLinkEnd(); 
           
            //Si el nodo destino del suffix-link de nodes[i] tena alguna transicin con actual
            //actualizamos nuestro suffix-link haciendo que nodes[nodoActual] apunte al destino
            //de dicha transicin
            int nodoDestino = getNode(linkEnd).hasTransition(currentWord);
            if (nodoDestino>-1) getNode(nodoActual).setLink(nodoActual, nodoDestino);
            else
            {
                //Aadimos la transicin hacia nosotros y llamamos recursivamente a oracle
                getNode(linkEnd).putTransition(new Transition(linkEnd, nodoActual, currentWord));
                numberOfTransitions++;
                recursiveOracle(linkEnd, nodoActual);
            }
        }
    }
    
    
}

