/*
 * Transition.java
 * 
 * Created on 18 de marzo de 2005, 15:16
 */

package oracle;

import java.lang.String;

/**
 * Transition define, mediante dos enteros y un String, un origen y destino de una transicin
 * que se realiza con la llegada de uno o varios carcteres. Una transicin entre los nodos
 * 4 y 8 de un grafo con valor 'A' se creara como una Transition(4, 8, "A");
 *
 * @author Alejandro Touza Paredes
 */
public class Transition {
    
    /** Origen de la transicin. */ 
    private int start;
    /** Destino de la transicin. */
    private int end;
    /** Valor de la transicin */    
    private String word;
    
    // CONSTRUCTORAS 
    /** Crea una nueva instancia de Transition con los valores por defecto. */
    public Transition() {
        start = -1;
        end = -1;
        word = "";
    }
    /**
     * Crea una nueva instancia de Transition con los valores pasados por parmetro.
     * @param start Origen de la transicin.
     * @param end Destino de la transicin.
     * @param word Valor de la transicin.
     */
    public Transition(int start, int end, String word) {
        this.start = start;
        this.end = end;
        this.word = word;
    }
    /**
     * Crea una nueva instancia de Transition copia de la Transition t.
     * @param t Transicin que se copiar.
     */
    public Transition(Transition t) {
        this.start = t.start;
        this.end = t.end;
        this.word = t.word;
    }
    
    // MODIFICADORAS 
    /** Actualiza end con el entero proporcionado.
     *  @param end Nuevo valor del destino de la transicin. */
    public void setEnd(int end){
        
        this.end = end;
    }
    /** Actualiza start con el entero proporcionado.
     *  @param start Nuevo valor del origen de la transicin. */
    public void setStart(int start){
        
        this.start = start;
    }
    /** Actualiza word con el String proporcionado.
     *  @param word Nuevo valor del word de la transicin. */
    public void setWord(String word){
        
        this.word = word;
    }
    /** Actualiza source, end y word con los valores proporcionados.
     *  @param start Nuevo valor del origen de la transicin.
     *  @param end Nuevo valor del destino de la transicin.
     *  @param word Nuevo valor del word de la transicin. */
    public void setValues(int start, int end, String word){
        
        this.start = start;
        this.end = end;
        this.word = word;
    }
    
    // CONSULTORAS
    /** Devuelve el parmetro source. 
     *  @return Valor de start */
    public int getStart(){
        
        return this.start;
    }
    /** Devuelve el parmetro end.
     *  @return Valor de end */
    public int getEnd(){
        
        return this.end;
    }
    /** Devuelve el parmetro word
     *  @return Valor de word */
    public String getWord(){
        
        return this.word;
    }
    /**
     * Devuelve cierto si la transicin t es igual a la que invoca la operacin.
     * @param t Transicin con la que se realiza la comparacin.
     * @throws java.lang.NullPointerException si t es nula.
     * @return <b>true</b> si la transicin es igual a t <b>false</b> en caso contrario
     */
    public boolean isEquals(Transition t) throws NullPointerException{
        return ( (this.start == t.start) && 
                 (this.end == t.end) && 
                 (this.word.equals(word)) );
    }
    /** Indica si la transicin t es nula.
     *  @return <b>true</b> si la transicin es nula <b>false</b> en caso conrario. */ 
    public boolean isNull(){
        return start==-1 && end==-1 && word.equals("");
    }
    
    // VARIAS
    /**
     * Realiza una copia de t en la transicin que invoca la operacin.
     * @param t Transicin de la que se realiza la copia.
     */
    public void copy(Transition t) {
        this.start = t.start;
        this.end = t.end;
        this.word = t.word;
    }
    /**
     * Devuelve un String con los valores de la transicin.
     * @return String con los valores de la transicin en formato (start) ---(word)---> (end).
     */
    public String print(){
        
        return ("(" + start + ") ---(" + word + ")---> (" + end + ")");
    }
}
